/* ftpq: ftp.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"


/* Make the FTP data connection, using the parameters returned by the
   PASV command (IP address, port high/low bytes). Return the socket
   descriptor. */

int data_connect (int a1, int a2, int a3, int a4, int p1, int p2) {

	int sockfd;
	char name[16];
	int port;

	sprintf (name, "%d.%d.%d.%d", a1, a2, a3, a4);
	port = (p1 * 256) + p2;

	sockfd = get_socket (name, port);

	return sockfd;
}


/* Connect to the named server/port, get the welcome message, and return
   the socket descriptor. */

int server_connect(char *name, int port) {

	int sockfd;

	sockfd = get_socket (name, port);

	if (sockfd != 0)
		get_response (sockfd);
  
	return sockfd;
}


/* Send a command to the FTP/SMTP server. If verbose mode is specified,
   the sent command is printed: in the case of the PASS command, the password
   is replaced with asterisks. If a reply is expected, get_response() (below)
   is called, and its return value is returned to our caller. */

int send_cmd (char *cmd, int sockfd, int resp) {

	int ctr, stars;

	if (cmd != NULL) {
		send (sockfd, cmd, strlen (cmd), 0);

		if (gv_verbose == VERBOSE) {
			if (strstr (cmd, "PASS ") == cmd) {
				printf ("-> PASS ");
				stars = strlen (cmd) - 6;
				for (ctr = 0; ctr < stars; ctr++) {
					printf ("*");
				}
				printf ("\n");
			}
			else
				printf ("-> %s", cmd);
		}
	}

	if (resp == NORESP)
		return OK;
	else
		return get_response (sockfd);
}


/* Read the response from the FTP server, put the last line into the global
   hostresp[] array (statically defined in ftpq.h) and return the response
   code. Servers can return multiple-line responses as multiple messages
   or as a single message including line breaks. See RFC959 for details of how
   the end of the message is denoted.
   This also works for SMTP responses, and is used by mailer.c */

int get_response (int sockfd) {

	int charctr = 0;
	char retcode[4];
	int firstcode = 0;
	enum {D2, D3, SEP, NL, WAIT, LAST, DONE} state = NL;
	int numbytes;

	retcode[3] = '\0';
	memset (hostresp, '\0', HOSTRESP);

	while (state != DONE) {
		if ((numbytes = recvfrom (sockfd, hostresp, HOSTRESP, 0, NULL, 0)) == -1)
			return 600;
		hostresp[numbytes] = '\0';
		charctr = 0;
		while (charctr < numbytes) {
			switch (state) {
			  case NL:
				if (gv_verbose == VERBOSE)
					printf ("<- ");
				if (hostresp[charctr] == '\n')
					state = NL;
				else if (isdigit (hostresp[charctr])) {
					retcode[0] = hostresp[charctr];
					state = D2;
				}
				else
					state = WAIT;
				break;
			  case D2:
				if (isdigit (hostresp[charctr])) {
					retcode[1] = hostresp[charctr];
					state = D3;
				}
				else if (hostresp[charctr] == '\n')
					state = NL;
				else
					state = WAIT;
				break;
			  case D3:
				if (isdigit (hostresp[charctr])) {
					retcode[2] = hostresp[charctr];
					state = SEP;
				}
				else if (hostresp[charctr] == '\n')
					state = NL;
				else
					state = WAIT;
				break;
			  case SEP:
				if (hostresp[charctr] == ' ') {
					if (firstcode == 0) {
						firstcode = atoi (retcode);
						state = LAST;
					}
					else if (firstcode != atoi (retcode))
						state = WAIT;
					else
						state = LAST;
				}
				else if (hostresp[charctr] == '-' && firstcode == 0) {
					firstcode = atoi (retcode);
					state = WAIT;
				}
				else if (hostresp[charctr] == '\n')
					state = NL;
				else
					state = WAIT;
				break;
			  case WAIT:
				if (hostresp[charctr] == '\n')
					state = NL;
				break;
			  case LAST:
				if (hostresp[charctr] == '\n')
					state = DONE;
				break;
			  case DONE:
				break;
			  default:
				printf ("Ouch - ftp state machine error!\n");
				exit (AB_CASEERR);
			}
			if (gv_verbose == VERBOSE)
				printf ("%c", hostresp[charctr]);
			charctr++;
		}
	}

	fflush (stdout);

	return firstcode;
}


/* category (response) categorizes an FTP response into one of the
   five response "types" listed in RFC959, or a default error category. */

int category (int response) {

	int reply = RESP_ERROR;

	if (response >= 600)
		reply = RESP_ERROR;
	else if (response >= 500)
		reply = NEG_PERM;
	else if (response >= 400)
		reply = NEG_TRANS;
	else if (response >= 300)
		reply = POS_INTER;
	else if (response >= 200)
		reply = POS_COMPLETE;
	else if (response >= 100)
		reply = POS_PRELIM;
	else
		reply = RESP_ERROR;

	return reply;
}
