/* ftpq: mailer.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"


/* Send a system message about a failed job to the SMTP port on the local
   host, directing the message to the owner of the job. Where the nature of
   the failure implies that the remainder of jobs on that login will be
   skipped, the jobs not processed will also be listed via a call to
   list_jobs () (below). */

int sendmail (struct server *this_server, struct login *this_login, struct job *this_job, int fault) {

	int sockfd;
	struct passwd *userdets = NULL;
	char *send = NULL;

	if (this_login == NULL)
		this_login = this_server->logins;

	if (this_job == NULL)
		this_job = this_login->jobs;

	if (this_job != NULL) {
		userdets = getpwuid (this_job->uid);
		if (userdets == NULL)
			return MAILERR;
	}

	sockfd = get_socket ("localhost", SMTP_PORT);

	if (sockfd == 0)
		return MAILERR;

	if (get_response (sockfd) != 220) {
		close (sockfd);
		return MAILERR;
	}

	if (send_cmd ("HELO localhost\r\n", sockfd, RESP) != 250) {
		close (sockfd);
		return MAILERR;
	}

	if (send_cmd ("MAIL FROM: ftpq@localhost\r\n", sockfd, RESP) != 250) {
		close (sockfd);
		return MAILERR;
	}

	send = memalloc (strlen (userdets->pw_name) + 21);
	sprintf (send, "RCPT TO: %s@localhost\r\n", userdets->pw_name);

	if (send_cmd (send, sockfd, RESP) != 250) {
		free (send);
		close (sockfd);
		return MAILERR;
	}

	free (send);

	if (send_cmd ("DATA\r\n", sockfd, RESP) != 354) {
		close (sockfd);
		return MAILERR;
	}

	send_cmd ("Subject: ftpq transmission error\r\n\r\n", sockfd, NORESP);
	send_cmd ("ftpq encountered an error trying to send one of your jobs\r\n", sockfd, NORESP);

	send = memalloc (27);
	sprintf (send, "Job: %5d ; Error: %5d\r\n", this_job->jobid, fault);
	send_cmd (send, sockfd, NORESP);
	free (send);

	switch (fault) {
	  case SEC_FAIL:
		send_cmd ("Security check failed: spool file creator doesn't own job file.\r\n", sockfd, NORESP);
		break;
	  case NOLOGIN:
		send_cmd ("Could not log in to server.\r\n", sockfd, NORESP);
		if (this_job->next != NULL) 
			list_jobs (this_job->next, sockfd);
		break;
	  case NOSERVER:
		send_cmd ("Could not contact server.\r\n", sockfd, NORESP);
		list_jobs (this_job, sockfd);
		break;
	  case NOCWDROOT:
		send_cmd ("Couldn't change to root directory on server.\r\n", sockfd, NORESP);
		list_jobs (this_job, sockfd);
		break;
	  case NOCWD:
		send_cmd ("Couldn't change to specified remote directory.\r\n", sockfd, NORESP);
		break;
	  case NOTYPEI:
		send_cmd ("Couldn't change to image type on server.\r\n", sockfd, NORESP);
		list_jobs (this_job, sockfd);
		break;
	  case NOMODES:
		send_cmd ("Couldn't select stream mode on server.\r\n", sockfd, NORESP);
		list_jobs (this_job, sockfd);
		break;
	  case NOPASV:
		send_cmd ("Couldn't negotiate passive mode with server.\r\n", sockfd, NORESP);
		list_jobs (this_job, sockfd);
		break;
	  case NOSTOR:
		send_cmd ("File transfer failed.\r\n", sockfd, NORESP);
		break;
	  default:
		send_cmd ("You shouldn't see this message, ever. Mail the author.\r\n", sockfd, NORESP);
	}

	send_cmd ("Run 'ftpq list' to view unsent jobs.\r\n", sockfd, NORESP);

	if (send_cmd (".\r\n", sockfd, RESP) != 250) {
		close (sockfd);
		return MAILERR;
	}

	if (send_cmd ("QUIT\r\n", sockfd, RESP) != 221) {
		close (sockfd);
		return MAILERR;
	}

	close (sockfd);

	return OK;
}


/* Walk through the job list listing other jobs not to be processed as a
   result of a previous job failure. */

int list_jobs (struct job *this_job, int sockfd) {

	char jobnumber[6];

	send_cmd ("This affects the following jobs, which will not be processed:\r\n\r\n", sockfd, NORESP);

	while (this_job != NULL) {
		sprintf (jobnumber, "%d", this_job->jobid);
		send_cmd (jobnumber, sockfd, NORESP);
		send_cmd (" : ", sockfd, NORESP);
		send_cmd (this_job->rdir, sockfd, NORESP);
		if (this_job->rdir[strlen (this_job->rdir) - 1] != '/')
			send_cmd ("/", sockfd, NORESP);
		send_cmd (this_job->rfile, sockfd, NORESP);
		send_cmd ("\r\n", sockfd, NORESP);
		this_job = this_job->next;
	}

	send_cmd ("\r\n", sockfd, NORESP);

	return OK;
}
